@@ -1,5 +1,5 @@
 /*
-  $Id: tokenize.h 13842 2012-01-02 16:32:58Z sloot $
+  $Id: tokenize.h 14285 2012-02-21 10:18:28Z mvgompel $
   $URL: https://ilk.uvt.nl/svn/sources/ucto/trunk/include/ucto/tokenize.h $
   Copyright (c) 2006 - 2012
   Tilburg University
@@ -322,10 +322,14 @@ namespace Tokenizer {
     bool xmlout;  
     bool xmlin;  
     bool passthru;
+    
+    int parCount;
 
     std::string settingsfilename;
     
-    std::string docid; //document ID (UTF-8), necessary for XML output  
+    std::string docid; //document ID (UTF-8), necessary for XML output 
+  private:
+    void sortRules( std::vector<Rule *>&, std::vector<UnicodeString>& );
   };
 
   template< typename T >
