@@ -1,5 +1,5 @@
 /*
-  $Id: tokenize.cxx 13916 2012-01-09 15:54:11Z sloot $
+  $Id: tokenize.cxx 14336 2012-02-27 10:38:57Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/ucto/trunk/src/tokenize.cxx $
   Copyright (c) 2006 - 2012
   Tilburg University
@@ -144,14 +144,14 @@ namespace Tokenizer {
     post = "";
     results.clear();
     if ( matcher ){
-      //      cerr << "start matcher [" << line << "]" << endl;
+      //  *Log(theErrLog) << "start matcher [" << line << "]" << endl;
       matcher->reset( line );
       if ( matcher->find() ){
-	// cerr << "matched " << folia::UnicodeToUTF8(line) << endl;
+	// *Log(theErrLog) << "matched " << folia::UnicodeToUTF8(line) << endl;
 	int start = -1;
 	int end = 0;
 	for ( int i=0; i <= matcher->groupCount(); ++i ){
-	  // cerr << "group " << i << endl;
+	  // *Log(theErrLog) << "group " << i << endl;
 	  u_stat = U_ZERO_ERROR;
 	  start = matcher->start( i, u_stat );
 	  if (!U_FAILURE(u_stat)){
@@ -163,19 +163,19 @@ namespace Tokenizer {
 	    break;
 	  if ( start > end ){
 	    pre = UnicodeString( line, end, start );
-	    // cerr << "found pre " << folia::UnicodeToUTF8(pre) << endl;
+	    // *Log(theErrLog) << "found pre " << folia::UnicodeToUTF8(pre) << endl;
 	  }
 	  end = matcher->end( i, u_stat );
 	  if (!U_FAILURE(u_stat)){
 	    results.push_back( UnicodeString( line, start, end - start ) );
-	    // cerr << "added result " << folia::UnicodeToUTF8( results[results.size()-1] ) << endl;
+	    // *Log(theErrLog) << "added result " << folia::UnicodeToUTF8( results[results.size()-1] ) << endl;
 	  }
 	  else
 	    break;
 	}
 	if ( end < line.length() ){
 	  post = UnicodeString( line, end );
-	  // cerr << "found post " << folia::UnicodeToUTF8(post) << endl;
+	  // *Log(theErrLog) << "found post " << folia::UnicodeToUTF8(post) << endl;
 	}
 	return true;
       }
@@ -318,7 +318,7 @@ namespace Tokenizer {
     paragraphsignal(true),sentencesignal(true),
     sentenceperlineoutput(false), sentenceperlineinput(false), 
     lowercase(false), uppercase(false), 
-    xmlout(false), passthru(false)
+    xmlout(false), passthru(false), parCount(0)
   {}
 
   string TokenizerClass::setInputEncoding( const std::string& enc ){
@@ -338,6 +338,7 @@ namespace Tokenizer {
     bool in_paragraph = false; //for XML
     bool done = false;
     bool bos = true;
+    parCount = 0;
     folia::Document doc( "id='" + docid + "'" );
     doc.addStyle( "type=\"text/xsl\" href=\"folia.xsl\"" );
     doc.declare( folia::AnnotationType::TOKEN, settingsfilename, "annotator='ucto', annotatortype='auto'" );
@@ -353,7 +354,8 @@ namespace Tokenizer {
       if ( (done) || (line.empty()) ){
 	signalParagraph();
 	numS = countSentences(true); //count full sentences in token buffer, force buffer to empty!
-      } else {
+      } 
+      else {
 	if ( passthru )
 	  passthruLine( line, bos );
 	else
@@ -400,134 +402,136 @@ namespace Tokenizer {
     if (element->isinstance(folia::Word_t) || element->isinstance(folia::TextContent_t)) return false;
     if (tokDebug >= 2) *Log(theErrLog) << "[tokenize] Processing FoLiA element " << element->id() << endl;
     if (element->hastext()) {
-	if (element->isinstance(folia::Paragraph_t)) {
-	    //tokenize paragraph: check for absence of sentences
-	    vector<folia::Sentence*> sentences = element->sentences();
-	    if (sentences.size() == 0) {
-		//no sentences yet, good
-		tokenize(element,true,false);
-		return true;
-	    } 
-	} else if ( (element->isinstance(folia::Sentence_t)) || (element->isinstance(folia::Head_t)) ) {
-	    //tokenize sentence: check for absence of words
+      if (element->isinstance(folia::Paragraph_t)) {
+	//tokenize paragraph: check for absence of sentences
+	vector<folia::Sentence*> sentences = element->sentences();
+	if (sentences.size() == 0) {
+	  //no sentences yet, good
+	  tokenize(element,true,false);
+	  return true;
+	} 
+      } else if ( (element->isinstance(folia::Sentence_t)) || (element->isinstance(folia::Head_t)) ) {
+	//tokenize sentence: check for absence of words
+	vector<folia::Word*> words = element->words();
+	if (words.size() == 0) {
+	  tokenize(element,false,true);
+	  return true;
+	} else {
+	  return false;
+	}
+      } else {
+	vector<folia::Paragraph*> paragraphs = element->paragraphs();
+	if (paragraphs.size() == 0) {
+	  vector<folia::Sentence*> sentences = element->sentences();
+	  if (sentences.size() == 0) {
 	    vector<folia::Word*> words = element->words();
 	    if (words.size() == 0) {
-		tokenize(element,false,true);
-		return true;
-	    } else {
-		return false;
+	      bool treat_as_paragraph = element->isinstance(folia::Event_t);
+	      tokenize(element,treat_as_paragraph,false);
+	      return true;			
 	    }
-	} else {
-	    vector<folia::Paragraph*> paragraphs = element->paragraphs();
-	    if (paragraphs.size() == 0) {
-		vector<folia::Sentence*> sentences = element->sentences();
-		if (sentences.size() == 0) {
-		    vector<folia::Word*> words = element->words();
-		    if (words.size() == 0) {
-			tokenize(element,false,false);
-			return true;			
-		    }
-		}
-	    }
-	    return false;
+	  }
 	}
+	return false;
+      }
     }
     //recursion step for other elements
     if (tokDebug >= 2) *Log(theErrLog) << "[tokenize] Processing children of FoLiA element " << element->id() << endl;
     for ( size_t i = 0; i < element->size(); i++) {
-	tokenize(element->index(i));
+      tokenize(element->index(i));
     }
     return false;
   }
   
   bool TokenizerClass::tokenize(folia::FoliaElement * element, bool root_is_paragraph, bool root_is_sentence) {
-        if (tokDebug >= 1) *Log(theErrLog) << "[tokenize] Processing FoLiA sentence" << endl;
-	UnicodeString line = element->stricttext() + " "  + explicit_eos_marker;
-	tokenizeLine(line);		
-	int numS = countSentences(true); //force buffer to empty
-	//ignore EOL data, we have by definition only one sentence:
-	bool in_par = false; //very ugly, I know
-	for (int i = 0; i < numS; i++) {
-	    int begin, end;
-	    if (!getSentence(i, begin, end)) throw uRangeError("Sentence index"); //should never happen
-	    if (tokDebug >= 1) *Log(theErrLog) << "[tokenize] Outputting sentence " << i << ", begin="<<begin << ",end="<< end << endl;
-	    outputTokensXML(element,begin,end,in_par,root_is_paragraph,root_is_sentence);
-	}
-	flushSentences(numS);	
-	if (numS > 0)
-	 return true;
-        else
-	 return false;
+    if (tokDebug >= 1) *Log(theErrLog) << "[tokenize] Processing FoLiA sentence" << endl;
+    UnicodeString line = element->stricttext() + " "  + explicit_eos_marker;
+    tokenizeLine(line);		
+    int numS = countSentences(true); //force buffer to empty
+    //ignore EOL data, we have by definition only one sentence:
+    bool in_par = false; //very ugly, I know
+    for (int i = 0; i < numS; i++) {
+      int begin, end;
+      if (!getSentence(i, begin, end)) throw uRangeError("Sentence index"); //should never happen
+      if (tokDebug >= 1) *Log(theErrLog) << "[tokenize] Outputting sentence " << i << ", begin="<<begin << ",end="<< end << endl;
+      outputTokensXML(element,begin,end,in_par,root_is_paragraph,root_is_sentence);
+    }
+    flushSentences(numS);	
+    if (numS > 0)
+      return true;
+    else
+      return false;
   }
   
   
   void TokenizerClass::tokenize( istream& IN, ostream& OUT) {
-      bool firstoutput = true;
-      bool in_paragraph = false; //for XML
-      bool done = false;
-      bool bos = true;
-      folia::Document doc( "id='" + docid + "'" );
-      if ( xmlout ){
-	doc.addStyle( "type=\"text/xsl\" href=\"folia.xsl\"" );
-	doc.declare( folia::AnnotationType::TOKEN, settingsfilename, "annotator='ucto', annotatortype='auto'" );
-	folia::FoliaElement *text = new folia::Text( "id='" + docid + ".text'" );
-	doc.append( text );
-      }
-      string line;      
-      do {	    
-	done = !getline( IN, line );
-	stripCR( line );
-	if ( sentenceperlineinput )
-	  line += string(" ") + folia::UnicodeToUTF8(explicit_eos_marker);
-	int numS;
-	if ( (done) || (line.empty()) ){
-	  signalParagraph();
-	  numS = countSentences(true); //count full sentences in token buffer, force buffer to empty!
-	} else {
-	  if ( passthru )
-	    passthruLine( line, bos );
-	  else
-	    tokenizeLine( line ); 
-	  numS = countSentences(); //count full sentences in token buffer	    
-	}			
-	if ( numS > 0 ) { //process sentences 
-	  if  (tokDebug > 0) *Log(theErrLog) << "[tokenize] " << numS << " sentence(s) in buffer, processing..." << endl;
-	  for (int i = 0; i < numS; i++) {
-	    int begin, end;
-	    if (!getSentence(i, begin, end)) {
-              if  (tokDebug > 0) *Log(theErrLog) << "[tokenize] ERROR: Sentence index " << i << " is out of range!!" << endl;
-	      throw uRangeError("Sentence index"); //should never happen
-	    }
-	    /* ******* Begin process sentence  ********** */
-	    if (tokDebug >= 1) *Log(theErrLog) << "[tokenize] Outputting sentence " << i << ", begin="<<begin << ",end="<< end << endl;
-	    if (xmlout) {
-	      outputTokensXML( doc, begin, end, in_paragraph );
-	    } else {
-	      outputTokens(OUT, begin, end, firstoutput );
-	      firstoutput = false;
-	    }	       
-	  }
-	  //clear processed sentences from buffer
-	  if  (tokDebug > 0) *Log(theErrLog) << "[tokenize] flushing " << numS << " sentence(s) from buffer..." << endl;
-	  flushSentences(numS);	    
-	} else {
-	  if  (tokDebug > 0) *Log(theErrLog) << "[tokenize] No sentences yet, reading on..." << endl;
-	}	
-      } while (!done);
-      if (xmlout) {
-	OUT << doc << endl;
+    bool firstoutput = true;
+    bool in_paragraph = false; //for XML
+    bool done = false;
+    bool bos = true;
+    folia::Document doc( "id='" + docid + "'" );
+    if ( xmlout ){
+      parCount = 0;
+      doc.addStyle( "type=\"text/xsl\" href=\"folia.xsl\"" );
+      doc.declare( folia::AnnotationType::TOKEN, settingsfilename, "annotator='ucto', annotatortype='auto'" );
+      folia::FoliaElement *text = new folia::Text( "id='" + docid + ".text'" );
+      doc.append( text );
+    }
+    string line;      
+    do {	    
+      done = !getline( IN, line );
+      stripCR( line );
+      if ( sentenceperlineinput )
+	line += string(" ") + folia::UnicodeToUTF8(explicit_eos_marker);
+      int numS;
+      if ( (done) || (line.empty()) ){
+	signalParagraph();
+	numS = countSentences(true); //count full sentences in token buffer, force buffer to empty!
       } else {
-	OUT << endl;
-      }
+	if ( passthru )
+	  passthruLine( line, bos );
+	else
+	  tokenizeLine( line ); 
+	numS = countSentences(); //count full sentences in token buffer	    
+      }			
+      if ( numS > 0 ) { //process sentences 
+	if  (tokDebug > 0) *Log(theErrLog) << "[tokenize] " << numS << " sentence(s) in buffer, processing..." << endl;
+	for (int i = 0; i < numS; i++) {
+	  int begin, end;
+	  if (!getSentence(i, begin, end)) {
+	    if  (tokDebug > 0) *Log(theErrLog) << "[tokenize] ERROR: Sentence index " << i << " is out of range!!" << endl;
+	    throw uRangeError("Sentence index"); //should never happen
+	  }
+	  /* ******* Begin process sentence  ********** */
+	  if (tokDebug >= 1) *Log(theErrLog) << "[tokenize] Outputting sentence " << i << ", begin="<<begin << ",end="<< end << endl;
+	  if (xmlout) {
+	    outputTokensXML( doc, begin, end, in_paragraph );
+	  } else {
+	    outputTokens(OUT, begin, end, firstoutput );
+	    firstoutput = false;
+	  }	       
+	}
+	//clear processed sentences from buffer
+	if  (tokDebug > 0) *Log(theErrLog) << "[tokenize] flushing " << numS << " sentence(s) from buffer..." << endl;
+	flushSentences(numS);	    
+      } else {
+	if  (tokDebug > 0) *Log(theErrLog) << "[tokenize] No sentences yet, reading on..." << endl;
+      }	
+    } while (!done);
+    if (xmlout) {
+      OUT << doc << endl;
+    } else {
+      OUT << endl;
+    }
   }
   
   UnicodeString xmlescape(UnicodeString s_in) {
-      UnicodeString s = s_in;      
-      s = s.findAndReplace("&","&amp;");
-      s = s.findAndReplace("\"","&quot;");
-      s = s.findAndReplace("<","&lt;");
-      s = s.findAndReplace(">","&gt;");
-      return s;
+    UnicodeString s = s_in;      
+    s = s.findAndReplace("&","&amp;");
+    s = s.findAndReplace("\"","&quot;");
+    s = s.findAndReplace("<","&lt;");
+    s = s.findAndReplace(">","&gt;");
+    return s;
   }
   
   void TokenizerClass::outputTokensXML( folia::Document& doc,
@@ -552,36 +556,32 @@ namespace Tokenizer {
     short quotelevel = 0;
     folia::FoliaElement *lastS = 0;
 
-    static int parCount = 0;    // Isn't this FATAL when multithreading?
+    //static int parCount = 0;    // Isn't this FATAL when multithreading?
     if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] parCount =" << parCount << endl;
-    if ((!root_is_paragraph) && (!root_is_sentence)) {
-	if ( !in_paragraph ){
-	  parCount = 0;
-	  if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] reset parCount to 0" << endl;
-	}
-	else {
-	  root = root->rindex(0);
-	  if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] root changed to " << root << endl;
-	}
+    
+       
+    if ((!root_is_paragraph) && (!root_is_sentence)) { 
+      if ( in_paragraph ){
+	root = root->rindex(0);
+	if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] root changed to " << root << endl;
+      }
     }
     
     if (root_is_sentence) {
-	lastS = root;
+      lastS = root;
     }
     
     for ( size_t i = begin; i <= end; i++) {
-	
+      
       if (((!root_is_paragraph) && (!root_is_sentence)) && ((tokens[i].role & NEWPARAGRAPH) || (!in_paragraph))) {	    
 	parCount++;
 	if ( in_paragraph )
 	  root = root->parent();
 	if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] Creating paragraph" << endl;
-	folia::FoliaElement *p = new folia::Paragraph( root->doc(),
-							  "id='" + root->doc()->id()
-							  + ".p." 
-							  + toString(parCount) 
-							  + "'" );
-	//	cerr << "created " << p << endl;
+	folia::KWargs args;
+	args["id"] = root->doc()->id() + ".p." +  toString(parCount);
+	folia::FoliaElement *p = new folia::Paragraph( root->doc(), args );
+	//	*Log(theErrLog) << "created " << p << endl;
 	root->append( p );
 	root = p;
 	quotelevel = 0;
@@ -590,30 +590,34 @@ namespace Tokenizer {
 	if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] End of quote";
 	quotelevel--;
 	root = root->parent();
-	//	cerr << "ENDQUOTE, terug naar " << root << endl;
+	//	*Log(theErrLog) << "ENDQUOTE, terug naar " << root << endl;
       }
       if ((tokens[i].role & BEGINOFSENTENCE) && (!root_is_sentence)) {
-	if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] Creating sentence" << endl;
-	folia::FoliaElement *s = new folia::Sentence( root->doc(),"generate_id='" + root->id() + "'" );
-	// cerr << "created " << s << endl;
+	if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] Creating sentence in '" << root->id() << "' ()" << endl;
+	folia::KWargs args;
+	args["generate_id"] = root->id();
+	folia::FoliaElement *s = new folia::Sentence( root->doc(), args );
+	// *Log(theErrLog) << "created " << s << endl;
 	root->append( s );
 	root = s;
 	lastS = s;
       }	
       if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] Creating word element for " << tokens[i].us << endl;
-      string args = "generate_id='" + lastS->id() + "',"  + " class= '" + folia::UnicodeToUTF8( *tokens[i].type ) + "'";
+      folia::KWargs args;
+      args["generate_id"] = lastS->id();
+      args["class"] = folia::UnicodeToUTF8( *tokens[i].type );
       if (tokens[i].role & NOSPACE) {
-	args += ", space='no'";
+	args["space"]= "no";
       }
       folia::FoliaElement *w = new folia::Word( root->doc(), args );
       w->settext( folia::UnicodeToUTF8( tokens[i].us ) );
-      //      cerr << "created " << w << " text= " <<  tokens[i].us << endl;
+      //      *Log(theErrLog) << "created " << w << " text= " <<  tokens[i].us << endl;
       root->append( w );
       if (tokens[i].role & BEGINQUOTE) {
 	if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] Creating quote element";
 	lastS = root;
 	folia::FoliaElement *q = new folia::Quote( root->doc(), "generate_id='" + root->id() + "'" );
-	//	cerr << "created " << q << endl;
+	//	*Log(theErrLog) << "created " << q << endl;
 	root->append( q );
 	root = q;
 	quotelevel++;
@@ -621,15 +625,12 @@ namespace Tokenizer {
       if ( (tokens[i].role & ENDOFSENTENCE) && (!root_is_sentence) ) {
 	if  (tokDebug > 0) *Log(theErrLog) << "[outputTokensXML] End of sentence";
 	root = root->parent();
-	//	cerr << "endsentence, terug naar " << root << endl;
+	//	*Log(theErrLog) << "endsentence, terug naar " << root << endl;
       }
       in_paragraph = true;
     }
   }
   
-  
-  
-  
   ostream& operator<<( ostream& os, const TokenRole& tok ){
     if ( tok & NOSPACE) os << "NOSPACE ";
     if ( tok & BEGINOFSENTENCE) os << "BEGINOFSENTENCE ";
@@ -650,9 +651,9 @@ namespace Tokenizer {
     for ( size_t i = begin; i <= end; i++) {
       if ((detectPar) && (tokens[i].role & NEWPARAGRAPH) && (!verbose) && (!firstoutput)) {
 	if (sentenceperlineoutput) {
-	  OUT << "\n";
+	  OUT << endl;
 	} else {
-	  OUT << "\n\n";
+	  OUT << endl << endl;
 	}
       }
       if (lowercase) {
@@ -675,10 +676,10 @@ namespace Tokenizer {
       if (quotelevel == 0) {
 	if (tokens[i].role & ENDOFSENTENCE) {
 	  if (verbose) {
-	    OUT << "\n";
+	    OUT << endl;
 	  } else {
 	    if (sentenceperlineoutput) {
-	      OUT << "\n";
+	      OUT << endl;
 	    } else {
 	      UnicodeString tmp = folia::UTF8ToUnicode( eosmark );
 	      OUT << " " + tmp;
@@ -804,21 +805,21 @@ namespace Tokenizer {
     short quotelevel = 0;
     begin = 0;
     for (int i = 0; i < size; i++) {
-	if (tokens[i].role & NEWPARAGRAPH) quotelevel = 0;
-	if (tokens[i].role & BEGINQUOTE) quotelevel++;
-        if (tokens[i].role & ENDQUOTE) quotelevel--;	
-	if ((tokens[i].role & BEGINOFSENTENCE) && (quotelevel == 0)) {
-	  begin = i;
+      if (tokens[i].role & NEWPARAGRAPH) quotelevel = 0;
+      if (tokens[i].role & BEGINQUOTE) quotelevel++;
+      if (tokens[i].role & ENDQUOTE) quotelevel--;	
+      if ((tokens[i].role & BEGINOFSENTENCE) && (quotelevel == 0)) {
+	begin = i;
+      }
+      if ((tokens[i].role & ENDOFSENTENCE) && (quotelevel == 0)) {
+	if (count == index) {
+	  end = i;
+	  if (!(tokens[begin].role & BEGINOFSENTENCE)) //sanity check
+	    tokens[begin].role |= BEGINOFSENTENCE;
+	  return true;
 	}
-        if ((tokens[i].role & ENDOFSENTENCE) && (quotelevel == 0)) {
-	  if (count == index) {
-	    end = i;
-	    if (!(tokens[begin].role & BEGINOFSENTENCE)) //sanity check
-	      tokens[begin].role |= BEGINOFSENTENCE;
-	    return true;
-	  }
-	  count++;
-	}	
+	count++;
+      }	
     }  
     return false;
   }
@@ -851,8 +852,8 @@ namespace Tokenizer {
 					    const bool firstoutput ) {
     int begin, end;
     if (!getSentence(i,begin,end)) {
-	throw uRangeError( "No sentence exists with the specified index: " 
-			   + toString( i ) );
+      throw uRangeError( "No sentence exists with the specified index: " 
+			 + toString( i ) );
     }
     
     //This only makes sense in non-verbose mode, force verbose=false
@@ -870,7 +871,7 @@ namespace Tokenizer {
     else
       return (tokens[tokens.size() - 1].role & ENDOFSENTENCE);
   }
-
+  
   bool TokenizerClass::resolveQuote(int endindex, const UnicodeString& open ) {
     //resolve a quote        
     int stackindex = -1;
@@ -882,7 +883,7 @@ namespace Tokenizer {
       if (beginindex > endindex) {
 	throw uRangeError( "Begin index for quote is higher than end index!" );
       }    
-
+      
       //We have a quote!
       
       //resolve sentences within quote, all sentences must be full sentences:
@@ -932,8 +933,7 @@ namespace Tokenizer {
       } else {
 	if (tokDebug >= 2) *Log(theErrLog) << "[resolveQuote] Quote can not be resolved, unbalanced sentences or subquotes within quote, skipping... (expectingend=" << expectingend << ",subquote=" << subquote << ")" << endl;
 	//something is wrong. Sentences within quote are not balanced, so we won't mark the quote.
-      }	
-      
+      }      
       
       //remove from stack (ok, granted, stack is a bit of a misnomer here)
       quotes.eraseAtPos( stackindex );
@@ -967,7 +967,7 @@ namespace Tokenizer {
       if (tokDebug > 1 )
 	*Log(theErrLog) << "[detectQuoteBounds] Standard double-quote (ambiguous) found @i="<< i << endl;
       if (!resolveQuote(i,c)) {
-	 if (tokDebug > 1 ) *Log(theErrLog) << "[detectQuoteBounds] Doesn't resolve, so assuming beginquote, pushing to stack for resolution later" << endl;
+	if (tokDebug > 1 ) *Log(theErrLog) << "[detectQuoteBounds] Doesn't resolve, so assuming beginquote, pushing to stack for resolution later" << endl;
 	quotes.push( i, c );
       }
     } else {
@@ -978,10 +978,10 @@ namespace Tokenizer {
       } else {
 	UnicodeString open = quotes.lookupClose( c );	
 	if ( !open.isEmpty() ) { // we have a closeing quote
-	 if (tokDebug > 1 ) *Log(theErrLog) << "[detectQuoteBounds] Closing quote found @i="<< i << ", attempting to resolve..." << endl;	      
+	  if (tokDebug > 1 ) *Log(theErrLog) << "[detectQuoteBounds] Closing quote found @i="<< i << ", attempting to resolve..." << endl;	      
 	  if (!resolveQuote(i, open )) // resolve the matching opening
-	   if (tokDebug > 1 ) *Log(theErrLog) << "[detectQuoteBounds] Unable to resolve" << endl;	      
-        //} else if (UnicodeString(c) == "―") {
+	    if (tokDebug > 1 ) *Log(theErrLog) << "[detectQuoteBounds] Unable to resolve" << endl;	      
+	  //} else if (UnicodeString(c) == "―") {
 	  //TODO: Implement
 	}
 
@@ -1023,42 +1023,37 @@ namespace Tokenizer {
 	    is_eos = true;
 	}
 	if (is_eos) {
-	  
-	  
 	  if ((detectQuotes) && (!quotes.emptyStack())) {
-	      
-	      if ((tokDebug > 1 )) 
-		*Log(theErrLog) << "[detectSentenceBounds] Preliminary EOS FOUND @i=" << i << endl;
-	      
-	      //if there are quotes on the stack, we set a temporary EOS marker, to be resolved later when full quote is found.
-	      tokens[i].role |= TEMPENDOFSENTENCE;
-	      //If previous token is also TEMPENDOFSENTENCE, it stops being so in favour of this one
-	      if ((i > 0) && (tokens[i-1].role & TEMPENDOFSENTENCE))
-		 tokens[i-1].role ^= TEMPENDOFSENTENCE;
+	    
+	    if ((tokDebug > 1 )) 
+	      *Log(theErrLog) << "[detectSentenceBounds] Preliminary EOS FOUND @i=" << i << endl;
+	    
+	    //if there are quotes on the stack, we set a temporary EOS marker, to be resolved later when full quote is found.
+	    tokens[i].role |= TEMPENDOFSENTENCE;
+	    //If previous token is also TEMPENDOFSENTENCE, it stops being so in favour of this one
+	    if ((i > 0) && (tokens[i-1].role & TEMPENDOFSENTENCE))
+	      tokens[i-1].role ^= TEMPENDOFSENTENCE;
 	  } else if (!sentenceperlineinput)  { //No quotes on stack (and no one-sentence-per-line input)
-	      sentencesignal = true;
-	      if ((tokDebug > 1 )) 
-		*Log(theErrLog) << "[detectSentenceBounds] EOS FOUND @i=" << i << endl;
-	      tokens[i].role |= ENDOFSENTENCE;
-	      //if this is the end of the sentence, the next token is the beginning of a new one
-	      if ((i + 1 < size) && !(tokens[i+1].role & BEGINOFSENTENCE))
-		tokens[i+1].role |= BEGINOFSENTENCE;
-		
-	      //if previous token is EOS and not BOS, it will stop being EOS, as this one will take its place
-	      if ((i > 0) && (tokens[i-1].role & ENDOFSENTENCE) && !(tokens[i-1].role & BEGINOFSENTENCE) ) {
-		tokens[i-1].role ^= ENDOFSENTENCE; 
-		if (tokens[i].role & BEGINOFSENTENCE) {
-		  tokens[i].role ^= BEGINOFSENTENCE;
-		}
-	      }   		
+	    sentencesignal = true;
+	    if ((tokDebug > 1 )) 
+	      *Log(theErrLog) << "[detectSentenceBounds] EOS FOUND @i=" << i << endl;
+	    tokens[i].role |= ENDOFSENTENCE;
+	    //if this is the end of the sentence, the next token is the beginning of a new one
+	    if ((i + 1 < size) && !(tokens[i+1].role & BEGINOFSENTENCE))
+	      tokens[i+1].role |= BEGINOFSENTENCE;
+	    
+	    //if previous token is EOS and not BOS, it will stop being EOS, as this one will take its place
+	    if ((i > 0) && (tokens[i-1].role & ENDOFSENTENCE) && !(tokens[i-1].role & BEGINOFSENTENCE) ) {
+	      tokens[i-1].role ^= ENDOFSENTENCE; 
+	      if (tokens[i].role & BEGINOFSENTENCE) {
+		tokens[i].role ^= BEGINOFSENTENCE;
+	      }
+	    }   		
 	  }	  	  
-
 	} else if (detectQuotes) {
 	  //check for other bounds
 	  detectQuoteBounds(i,c);
-	}	
-
-	
+	}
       }
     }
   }    
@@ -1153,8 +1148,8 @@ namespace Tokenizer {
       }   
     }
     if (sentenceperlineinput) {
-	tokens[0].role |= BEGINOFSENTENCE;
-	tokens[tokens.size() - 1].role |= ENDOFSENTENCE;
+      tokens[0].role |= BEGINOFSENTENCE;
+      tokens[tokens.size() - 1].role |= ENDOFSENTENCE;
     }
   }
 
@@ -1165,7 +1160,7 @@ namespace Tokenizer {
       uinputstring = UnicodeString( s.c_str(), s.length(), inputEncoding.c_str() );
     } catch ( exception &e) {
       throw uCodingError( "Unexpected character found in input. " + string(e.what() )
-			   + "Make sure input is valid UTF-8!" );
+			  + "Make sure input is valid UTF-8!" );
     }
     if ( uinputstring.isBogus() ){
       throw uCodingError( "string decoding failed: (invalid inputEncoding '" 
@@ -1239,18 +1234,18 @@ namespace Tokenizer {
 	      }
 	      if (!tokens.empty()) {
 		if (tokDebug >= 2) 
-		  *Log(theErrLog) << "[tokenizeLine] Assigned EOS\n";
+		  *Log(theErrLog) << "[tokenizeLine] Assigned EOS" << endl;
 		tokens[tokens.size() - 1].role |= ENDOFSENTENCE;
 	      }			
 	    }
 	  }			    
 	  if ((word.length() > 0) && (expliciteosfound == -1)) {	    
 	    if (!tokenizeword) {	      
-		//single character or nothing tokenisable found, so no need to tokenize anything
-		if (tokDebug >= 2)
-		  *Log(theErrLog) << "[tokenizeLine] Word ok, no need for further tokenisation for: ["
-				  << folia::UnicodeToUTF8( word ) << "]" << endl;;
-		tokens.push_back( Token( &type_word, word ) );
+	      //single character or nothing tokenisable found, so no need to tokenize anything
+	      if (tokDebug >= 2)
+		*Log(theErrLog) << "[tokenizeLine] Word ok, no need for further tokenisation for: ["
+				<< folia::UnicodeToUTF8( word ) << "]" << endl;;
+	      tokens.push_back( Token( &type_word, word ) );
 	    } else {
 	      if (tokDebug >= 2)
 		*Log(theErrLog) << "[tokenizeLine] Further tokenisation necessary for: [" 
@@ -1292,7 +1287,7 @@ namespace Tokenizer {
     matches.clear();
     pre = "";
     post = "";
-    //    cerr << "\nrule " << *this << endl;
+    //    *Log( theErrLog) << endl << "rule " << *this << endl;
     if ( regexp && regexp->match_all( line, pre, post ) ){
       int num = regexp->NumOfMatches();
       if ( num >=1 ){
@@ -1317,10 +1312,10 @@ namespace Tokenizer {
 	*Log(theErrLog) << "   [tokenizeWord] Found explicit EOS marker" << endl;
       if (!tokens.empty()) {
 	if (tokDebug >= 2) 
-	  *Log(theErrLog) << "   [tokenizeWord] Assigned EOS\n";
+	  *Log(theErrLog) << "   [tokenizeWord] Assigned EOS" << endl;
 	tokens[tokens.size() - 1].role |= ENDOFSENTENCE;
       } else {
-	*Log(theErrLog) << "[WARNING] Found explicit EOS marker by itself, this will have no effect!\n"; 
+	*Log(theErrLog) << "[WARNING] Found explicit EOS marker by itself, this will have no effect!" << endl; 
       }
       return;
     }
@@ -1578,7 +1573,8 @@ namespace Tokenizer {
     }
   }
 
-  void sortRules( vector<Rule *>& rules, vector<UnicodeString>& sort ){
+  void TokenizerClass::sortRules( vector<Rule *>& rules, 
+				  vector<UnicodeString>& sort ){
     // *Log(theErrLog) << "rules voor sort : " << endl;
     // for ( size_t i=0; i < rules.size(); ++i ){
     //   *Log(theErrLog) << "rule " << i << " " << *rules[i] << endl;
@@ -1597,14 +1593,14 @@ namespace Tokenizer {
 	  ++it;
 	}
 	if ( !found ){
-	  cerr << "RULE-ORDER specified for undefined RULE '" 
-	       << folia::UnicodeToUTF8( sort[i] ) << "'" << endl;
+	  *Log(theErrLog) << "RULE-ORDER specified for undefined RULE '" 
+			  << folia::UnicodeToUTF8( sort[i] ) << "'" << endl;
 	}
       }
       vector<Rule*>::iterator it = rules.begin();
       while ( it != rules.end() ){
-	cerr << "NU RULE-ORDER specified for RULE '" 
-	     << folia::UnicodeToUTF8((*it)->id) << "'" << endl;
+	*Log(theErrLog) << "NU RULE-ORDER specified for RULE '" 
+			<< folia::UnicodeToUTF8((*it)->id) << "'" << endl;
 	result.push_back( *it );
 	++it;
       }
@@ -1693,6 +1689,9 @@ namespace Tokenizer {
 	    mode = getMode( line );
 	  }
 	  else {
+	    if ( line[0] == '\\' && line.length() > 1 && line[1] == '[' ){
+	      line = UnicodeString( line, 1 );
+	    }
 	    switch( mode ){
 	    case RULES: {
 	      const int splitpoint = line.indexOf("=");
@@ -1818,7 +1817,7 @@ namespace Tokenizer {
     //TODO: Make order dynamically configurable?
 
     if (!ordinal_pattern.isEmpty()){
-      rules.insert(rules.begin(), new Rule("NUMBER-ORDINAL", "\\p{N}+-?(?:" + ordinal_pattern + ")(?i)(?:\\Z|\\P{L})")); 
+      rules.insert(rules.begin(), new Rule("NUMBER-ORDINAL", "\\p{N}+-?(?:" + ordinal_pattern + ")(?:\\Z|\\P{Lu}|\\P{Ll})")); 
       // NB: (?i) is not used for the whole expression because of icu bug 8824
       //     see http://bugs.icu-project.org/trac/ticket/8824
       //     If you want mixed case Ordinals, you have to enumerate them all
@@ -1832,22 +1831,22 @@ namespace Tokenizer {
       // was case insensitive, but seems a bad idea
     }
     if (!token_pattern.isEmpty()){
-      rules.insert(rules.begin(), new Rule("WORD-TOKEN", "(?i)^(" + token_pattern + ")(?:\\p{P}*)?$")); // (?:\\Z|\\P{L})")); 
+      rules.insert(rules.begin(), new Rule("WORD-TOKEN", "^(" + token_pattern + ")(?:\\p{P}*)?$"));
     }    
     if (!withprefix_pattern.isEmpty()){
-      rules.insert(rules.begin(), new Rule("WORD-WITHPREFIX", "(?i)(?:\\A|[^\\p{L}\\.])(?:" + withprefix_pattern + ")\\p{L}+")); 
+      rules.insert(rules.begin(), new Rule("WORD-WITHPREFIX", "(?:\\A|[^\\p{Lu}\\.]|[^\\p{Ll}\\.])(?:" + withprefix_pattern + ")\\p{L}+")); 
     }
     if (!withsuffix_pattern.isEmpty()){
-      rules.insert(rules.begin(), new Rule("WORD-WITHSUFFIX", "((?:\\p{Lu}|\\p{Ll})+(?:" + withsuffix_pattern + "))(?i)(?:\\Z|\\P{L})")); 
+      rules.insert(rules.begin(), new Rule("WORD-WITHSUFFIX", "((?:\\p{Lu}|\\p{Ll})+(?:" + withsuffix_pattern + "))(?:\\Z|\\P{Lu}|\\P{Ll})")); 
       // NB: (?:\\p{Lu}|\\p{Ll}) is used because of icu bug 8824
       //     see http://bugs.icu-project.org/trac/ticket/8824
       //     normally (?i) could be used in front and (\\p{L}) would do.
     }
     if (!prefix_pattern.isEmpty()){
-      rules.insert(rules.begin(), new Rule("PREFIX", "(?i)(?:\\A|[^\\p{L}\\.])(" + prefix_pattern + ")(\\p{L}+)")); 
+      rules.insert(rules.begin(), new Rule("PREFIX", "(?:\\A|[^\\p{Lu}\\.]|[^\\p{Ll}\\.])(" + prefix_pattern + ")(\\p{L}+)")); 
     }
     if (!suffix_pattern.isEmpty()){
-      rules.insert(rules.begin(), new Rule("SUFFIX", "(?i)(\\p{L}+)(" + suffix_pattern + ")(?:\\Z|\\P{L})")); 
+      rules.insert(rules.begin(), new Rule("SUFFIX", "(\\p{Lu}|\\p{Ll}+)(" + suffix_pattern + ")(?:\\Z|\\P{L})")); 
     }
     //rules.insert(rules.begin(), new Rule("EOSMARKER", "(?:.*)?(" + *explicit_eos_marker + ")(?:.*)?")); 
 
@@ -1856,7 +1855,7 @@ namespace Tokenizer {
   }
   
   bool TokenizerClass::init( const string& fname ){
-    *Log(theErrLog) << "Initiating tokeniser...\n";
+    *Log(theErrLog) << "Initiating tokeniser..." << endl;
     if (!readsettings( fname ) ) {
       throw uConfigError( "Cannot read Tokeniser settingsfile "+ fname );
       return false;
